<?php 
/* This file is part of "filebrowser".
 *    Copyright 2009, arNuméral
 *    Author : Yoran Brault
 *    eMail  : yoran.brault@bad_arnumeral.fr (remove bad_ before sending an email)
 *    Site   : http://www.arnumeral.fr
 *
 * "filebrowser" is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * "filebrowser" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with "filebrowser"; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

 function filebrowser_callback_create_thumbnails($node) {
   static $thumbnailers;
   if (!$thumbnailers) {
      $thumbnailers = module_implements("filebrowser_thumbnailer");
   }
   $jobs=array();
   if (count($thumbnailers) != 0) {
      foreach ($thumbnailers as $thumbnailer) {
         if ($node->$thumbnailer->enabled) {
            $result=module_invoke($thumbnailer, "filebrowser_prepare_thumbnailer", $node->file_listing, $node->$thumbnailer);
            if ($result) {
               $jobs=array_merge($jobs, $result);
            }
         }
      }
   }
   $operations=array();
   foreach ($jobs as $job) {
   	$operations[]=array('_filebrowser_create_thumbnails', $job);
   }
   if (count($operations)>0) {
      $batch = array(
      'title' => t('Creating thumbnails'),
      'operations' => $operations,
      'file' => drupal_get_path("module", "filebrowser")."/filebrowser.pages.inc"
      );
      batch_set($batch);
      batch_process("node/".$node->nid);
   } else {
      return t("All thumbnails are up to date");
   }
}
 
function _filebrowser_create_thumbnails($callback, $arguments, &$context) {
	if (!isset($context['sandbox']['progress'])) {
		$context['sandbox']['progress'] = 0;
		$context['sandbox']['max'] = 1;
	}
	call_user_func_array($callback, $arguments);
	$context['sandbox']['progress']++;
//	$context['message']=t('processing @file (@res size)', array('@file'=>$file[0], '@size'=>$file[2]));
//	if ($context['sandbox']['progress']!=$context['sandbox']['max']) {
		$context['finished'] = 1; //$context['sandbox']['progress'] / $context['sandbox']['max'];
//	}
} 
/**
 * Callback for filebrowser_download/%node menu.
 */
function filebrowser_callback_download($node) {
   if (!_filebrowser_can_download_file($node)) {
      drupal_access_denied();
      exit ();
   }

   $match = array ();
   foreach ($node->file_listing as $file_name => $file_data) {
      if ($node->download_file_name == $file_data['name']) {
         $match = $file_data;
         break;
      }
   }
   if (empty ($match)) {
      drupal_not_found();
      exit ();
   }
   _file_browser_send_file($match);
   exit ();
}

/**
 * Callback for filebrowser_download_archive/%node menu.
 */
function filebrowser_callback_download_archive($node) {
   if (!_filebrowser_can_download_archive($node)) {
      drupal_access_denied();
      exit ();
   }

   if (!function_exists("zip_open")) {
      return t("No ZIP support found in PHP installation, please contact your administrator");
   }
   $zip = new ZipArchive();
   $match['name'] = $node->download_file_name;
   $match['mime-type'] = 'application/zip';
   $match['path'] = file_directory_temp() . '/' . $match['name'];
   if (file_exists($match['path'])) {
      unlink($match['path']);
   }
   if ($zip->open($match['path'], ZIPARCHIVE :: CREATE) === TRUE) {
      foreach ($node->file_listing as $file_name => $file_data) {
         if ($file_data['kind'] == 0) {
            $zip->addFile($file_data['path'], $file_name);
         }
      }
      $zip->close();
   } else {
      return t("Unable to create temporary zip file '@file'", array (
      file => $match['name']
      ));
   }
   $match['size'] = filesize($match['path']);
   _file_browser_send_file($match);
   exit ();
}


